*, *:before, *:after {
  box-sizing: border-box;
}

/* Re-apply defaults after reset */
strong {
  font-weight: bold;
}

em {
  font-style: italic;
}

pre, code {
  font-family: monospace;
}

/* Not using body in case something like background image is needed to cover the entire screen  */
#fullpage {
  max-width: 63em;
  margin: auto;
}

article {
  padding: 10px 1em;
  margin: 0 5% 10px;
}

img {
  max-width: 100%;
}

body {
  font-size: 100%;
  font-family: "Helvetica Neue",Helvetica,Arial,sans-serif;
}

h1 {
  font-size: 2em;
}

h2 {
  font-size: 1.5em;
}

h1, h2 {
  font-weight: bold;
  margin: 5px 0;
}

p {
  font-size: 1em;
  margin: 5px 0;
  line-height: 1.2;
}

@media screen and (min-width: 25em) {
  article {
    font-size: 1.1em;
  }
}

@media screen and (min-width: 25em) {
  article {
    font-size: 1.2em;
  }
}

body {
  hyphens: auto;
}

code {
  hyphens: none;
}

body {
  word-wrap: break-word;
  overflow-wrap: break-word;
}

h1, h2, h3, h4, h5, h6, p {
  margin: 1em 0;
}

nav ul {
  display: flex;
  flex-direction: column;
  list-style-type: none;
  padding: 0;
  width: 100%;
  margin: 0 auto;
}

nav ul li {
  border: 1px solid #666;
  background-color: #eee;
  flex-grow: 1;
  text-align: center;
}

nav ul li a {
  display: block;
  text-decoration: none;
  font-size: 1.2em;
  padding: 10px 1em;
}

/* < 30 : One column menu */

/* 30 <= x < 44 : Two column menu */
@media only screen and (min-width: 30em) and (max-width: 44em) {
  nav ul {
    flex-direction: row;
    flex-wrap: wrap;
  }
  nav ul li {
    flex-basis: 50%;
  }
}

/* >= 44 : One Row menu */
@media only screen and (min-width: 44em) {
  nav ul {
    flex-direction: row;
  }
}

/* Footer */
footer {
  padding-top: 2em;
}

footer ul {
  display: flex;
  flex-direction: column;
  list-style-type: none;
  padding: 0;
  margin: 25px 0 0;
}

footer ul li {
  border-top: 1px solid #666;
  padding: 10px 1em;
  margin: 3px 0 0;
  text-align: center;
}

footer ul li a {
  text-decoration: none;
}

@media only screen and (min-width: 50em) {
  footer ul {
    margin: 0em;
    flex-direction: row; 
    flex-wrap: wrap;
    justify-content: center;
  }
  footer ul li {
      flex-grow: 1;
      border-top: none;
      margin: 0em;
      padding: 0em;
    }
}

.post-preview-link {
  text-decoration: none;
  color: black;
}

.post-preview-link:visited {
  color :black;
}

.post-title:hover {
  color: blue;
}

li.active {
  font-weight: bold;
}
