{% extends "_layout.html" %}

{% block title %}{{ month_string }} {{ day }}, {{ year }}{% endblock title %}

{% block content %}
  <article>
  <h1>Posts from {{ month_string }}, {{ day }} {{ year }}</h1>
  <hr class="small">
  {% for post in posts %}
    {% with show_text=False %}
      {% include "_list.html" %}
    {% endwith %}
  {% endfor %}
  </article>
{% endblock content %}
