{% extends "_layout.html" %}

{% block title %}{{ tag }}{% endblock title %}

{% block content %}
  <article>
    <span class="subheading">{% block site_description %}{% endblock site_description %}</span>
    {% for post in posts %}
      {% with show_text=False %}
        {% include "_list.html" %}
      {% endwith %}
    {% endfor %}
  </article>
{% endblock content %}
