{% extends "_layout.html" %}

{% block title %}Tags{% endblock title %}

{% block content %}
  <article>
  <h1>All Tags</h1>

  {% for tag, count in tags.items() %}
  <div class="post-preview">
      <a href="{{ url_for('tag', tag=tag) }}">
          <h2 class="post-title" style="font-size: {{ 1+0.1*count }}em">
              {{ tag }}
          </h2>
      </a>
  </div>
  <hr>
  {% endfor %}
  </article>
{% endblock content %}
