{% extends "_layout.html" %}

{% block title %}Home{% endblock title %}

{% block meta_tags %}
  {{ super() }}
  <meta property="og:url" content="{{ SITE_URL }}">
  <meta property="og:title" content="{{ SITE_NAME }}">
  {% if SITE_DESCRIPTION %}
    <meta name="description" content="{{ SITE_DESCRIPTION }}">
    <meta property="og:description" content="{{ SITE_DESCRIPTION }}">
  {% endif %}
  {% if SITE_LOGO %}
    <meta property="og:image" content="{{ url_for('static', filename=SITE_LOGO, _external=True) }}">
  {% endif %}
{% endblock meta_tags %}

{% block content %}
  <article>
  {% for post in posts %}
    {% with show_text=True %}
      {% include "_list.html" %}
    {% endwith %}
  {% endfor %}
  </article>
{% endblock content %}
